@@ -42,6 +42,24 @@
 #include "swf_tag.h"
 #include "swf_object.h"
 
+#define SWFED_VERSION "0.53"
+
+#define get_zend_hash_value_long(table, key, value) do { \
+        zval **tmp = NULL; \
+        if (zend_hash_find(table, key, sizeof(key), (void**)&tmp) == SUCCESS) { \
+            convert_to_long_ex(tmp); \
+            value = Z_LVAL_PP(tmp); \
+        } \
+    } while (0);
+
+#define get_zend_hash_value_boolean(table, key, value) do { \
+        zval **tmp = NULL; \
+        if (zend_hash_find(table, key, sizeof(key), (void**)&tmp) == SUCCESS) { \
+            convert_to_boolean_ex(tmp); \
+            value = Z_LVAL_PP(tmp); \
+        } \
+    } while (0);
+
 /* If you declare any globals in php_swfed.h uncomment this:
 ZEND_DECLARE_MODULE_GLOBALS(swfed)
 */
@@ -108,8 +126,6 @@ zend_function_entry swfed_functions[] = {
 };
 /* }}} */
 
-#define SWFED_VERSION "0.50"
-
 /* {{{ swfed_module_entry
  */
 zend_module_entry swfed_module_entry = {
@@ -284,7 +300,11 @@ PHP_METHOD(swfed, __construct) {
     if (swf == NULL) {
         php_error_docref(NULL TSRMLS_CC, E_ERROR, "Couldn't create swf object");
     }
+#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION > 3) || (PHP_MAJOR_VERSION > 5)
     ret = zend_list_insert(swf, le_swfed TSRMLS_CC);
+#else
+    ret = zend_list_insert(swf, le_swfed);
+#endif
     object_init_ex(getThis(), swfeditor_ce);
     add_property_resource(getThis(), "swfed", ret);
     zend_list_addref(ret);
@@ -655,7 +675,7 @@ PHP_METHOD(swfed, getTagDataByCID) {
         fprintf(stderr, "getTagDataByCID: Can't get_tagdata_bycid\n");
         RETURN_FALSE;
     }
-    RETURN_STRINGL(data_ref, data_len, 1);
+    RETURN_STRINGL((char *) data_ref, data_len, 1);
 }
 
 PHP_METHOD(swfed, replaceTagDataByCID) {
@@ -698,7 +718,7 @@ PHP_METHOD(swfed, getTagContentsByCID) {
         fprintf(stderr, "getTagContentsByCID: Can't get_tagcontents_bycid\n");
         RETURN_FALSE;
     }
-    RETURN_STRINGL(data_ref, data_len, 1);
+    RETURN_STRINGL((char *)data_ref, data_len, 1);
 }
 
 PHP_METHOD(swfed, replaceTagContentsByCID) {
@@ -793,7 +813,7 @@ PHP_METHOD(swfed, getShapeData) {
     new_buff = emalloc(data_len);
     memcpy(new_buff, data, data_len);
     free(data);
-    RETURN_STRINGL(new_buff, data_len, 0);
+    RETURN_STRINGL((char *)new_buff, data_len, 0);
 }
 
 PHP_METHOD(swfed, replaceShapeData) {
@@ -1011,6 +1031,9 @@ PHP_METHOD(swfed, replacePNGData) {
     char *data = NULL;
     int data_len = 0;
     int image_id = 0;
+    zval *opts = NULL;
+    HashTable *opts_table = NULL;
+    int rgb15 = -1;
     swf_object_t *swf;
     int result = 0;
     switch (ZEND_NUM_ARGS()) {
@@ -1018,15 +1041,23 @@ PHP_METHOD(swfed, replacePNGData) {
         WRONG_PARAM_COUNT;
         RETURN_FALSE; /* XXX */
       case 2:
-        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ls", &image_id, &data, &data_len) == FAILURE) {
+      case 3:
+        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ls|a", &image_id, &data, &data_len, &opts) == FAILURE) {
             RETURN_FALSE;
         }
         break;
     }
+    if (opts) {
+        opts_table = Z_ARRVAL_P(opts);
+        get_zend_hash_value_boolean(opts_table, "rgb15",  rgb15);
+    }
+    
     swf = get_swf_object(getThis() TSRMLS_CC);
+
     result = swf_object_replace_pngdata(swf, image_id,
                                         (unsigned char *)data,
-                                        (unsigned long) data_len);
+                                        (unsigned long) data_len,
+                                        rgb15);
     if (result) {
         RETURN_FALSE;
     }
@@ -1065,73 +1096,46 @@ PHP_METHOD(swfed, replaceGIFData) {
 #endif /* HAVE_GIF */
 }
 
-
-#define get_zend_hash_value_long(table, key, value) do { \
-        zval **tmp = NULL; \
-        if (zend_hash_find(table, key, sizeof(key), (void**)&tmp) == SUCCESS) { \
-            convert_to_long_ex(tmp); \
-            value = Z_LVAL_PP(tmp); \
-        } \
-    } while (0);
-
 PHP_METHOD(swfed, replaceBitmapData) {
     char *data = NULL, *alpha_data = NULL;
     int data_len = 0 , alpha_data_len = 0;
+    zval *arg1 = NULL, *arg4 = NULL;
     int image_id = 0;
-    zval *image_info = NULL;
     HashTable *image_info_table = NULL;
+    int width = -1, height = -1;
+    int red = -1, green = -1, blue = -1;
+    HashTable *opts_table = NULL;
+    int without_converting = 0;
+    int rgb15 = -1;
     swf_object_t *swf = NULL;
     int result = 0;
     int bitmap_format;
-    int width = -1, height = -1;
-    int red = -1, green = -1, blue = -1;
-    int without_converting = 0;
     switch (ZEND_NUM_ARGS()) {
       default:
         WRONG_PARAM_COUNT;
         RETURN_FALSE; /* XXX */
       case 2:
-          if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET,
-                                       ZEND_NUM_ARGS() TSRMLS_CC, "ls", &image_id, &data, &data_len) == SUCCESS) {
-              ; // OK
-          } else if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET,
-                                              ZEND_NUM_ARGS() TSRMLS_CC, "as", &image_info, &data, &data_len) == SUCCESS) {
-              ; // OK
-          } else {
-              php_error(E_WARNING, "%s() expects parameter 1 to be long or array given", get_active_function_name(TSRMLS_C));
-              
-              RETURN_FALSE;
-          }
-          break;
-    case 3:
+      case 3:
+      case 4:
+      case 5:
         if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET,
-                                     ZEND_NUM_ARGS() TSRMLS_CC, "lss", &image_id, &data, &data_len, &alpha_data, &alpha_data_len) == SUCCESS) {
-            ; // OK
-        } else if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET,
-                                            ZEND_NUM_ARGS() TSRMLS_CC, "ass", &image_info, &data, &data_len, &alpha_data, &alpha_data_len) == SUCCESS) {
+                                     ZEND_NUM_ARGS() TSRMLS_CC, "zs|sz", &arg1, &data, &data_len, &alpha_data, &alpha_data_len, &arg4) == SUCCESS) {
             ; // OK
         } else {
-            php_error(E_WARNING, "%s() expects parameter 1 to be long or array given", get_active_function_name(TSRMLS_C));
-            RETURN_FALSE;
-        }
-        break;
-    case 4:
-        if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET,
-                                     ZEND_NUM_ARGS() TSRMLS_CC, "lssb", &image_id, &data, &data_len, &alpha_data, &alpha_data_len, &without_converting) == SUCCESS) {
-            ; // OK
-        } else if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET,
-                                            ZEND_NUM_ARGS() TSRMLS_CC, "assb", &image_info, &data, &data_len, &alpha_data, &alpha_data_len, &without_converting) == SUCCESS) {
-            ; // OK
-        } else {
-            php_error(E_WARNING, "%s() expects parameter 1 to be long or array given", get_active_function_name(TSRMLS_C));
+            php_error(E_WARNING, "%s() expects parameter 2 to be string given", get_active_function_name(TSRMLS_C));
             RETURN_FALSE;
         }
         break;
     }
     swf = get_swf_object(getThis() TSRMLS_CC);
-    
-    if (image_info) {
-        image_info_table = Z_ARRVAL_P(image_info);
+
+    if (Z_TYPE_P(arg1) != IS_ARRAY) { // image_id (integer)
+        if (Z_TYPE_P(arg1) != IS_LONG) {
+            convert_to_long(arg1);
+        }
+        image_id = (int) Z_LVAL_P(arg1);
+    } else { // or image_info (array)
+        image_info_table = Z_ARRVAL_P(arg1);
         get_zend_hash_value_long(image_info_table, "width",  width);
         get_zend_hash_value_long(image_info_table, "height", height);
         get_zend_hash_value_long(image_info_table, "red",    red);
@@ -1145,7 +1149,18 @@ PHP_METHOD(swfed, replaceBitmapData) {
             RETURN_FALSE;
         }
     }
-
+    if (arg4 != NULL) {
+        if (Z_TYPE_P(arg4) != IS_ARRAY) { // without_converting (boolean)
+            if (Z_TYPE_P(arg4) != IS_BOOL) {
+                convert_to_boolean(arg4);
+            }
+            without_converting = (int) Z_LVAL_P(arg4);
+        } else { // or opts (array)
+            opts_table = Z_ARRVAL_P(arg4);
+            get_zend_hash_value_boolean(opts_table, "without_converting",  without_converting);
+            get_zend_hash_value_boolean(opts_table, "rgb15", rgb15);
+        }
+    }
     bitmap_format = detect_bitmap_format((unsigned char*) data, data_len);
     if (without_converting) { // for v8 JPEG Tag
         switch (bitmap_format) {
@@ -1176,7 +1191,8 @@ PHP_METHOD(swfed, replaceBitmapData) {
         case BITMAP_UTIL_FORMAT_PNG:
             result = swf_object_replace_pngdata(swf, image_id,
                                                 (unsigned char *)data,
-                                                (unsigned long) data_len);
+                                                (unsigned long) data_len,
+                                                rgb15);
             
             break;
         case BITMAP_UTIL_FORMAT_GIF:
@@ -1190,7 +1206,6 @@ PHP_METHOD(swfed, replaceBitmapData) {
             RETURN_FALSE;
         }
     }
-    
     if (result) {
         RETURN_FALSE;
     }
